#!/usr/bin/env node
// noinspection XmlDeprecatedElement,HtmlDeprecatedTag
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var os_1 = __importDefault(require("os"));
var promises_1 = __importDefault(require("fs/promises"));
var commander_1 = require("commander");
var bonjour_1 = __importDefault(require("bonjour"));
var Destination_1 = __importDefault(require("./Destination"));
var ScanJobSettings_1 = __importDefault(require("./ScanJobSettings"));
var HPApi_1 = __importDefault(require("./HPApi"));
var JpegUtil_1 = __importDefault(require("../src/JpegUtil"));
var PathHelper_1 = __importDefault(require("./PathHelper"));
var ScanContent_1 = require("./ScanContent");
var program = new commander_1.Command();
function delay(t) {
    return new Promise(function (resolve) {
        setTimeout(resolve, t);
    });
}
function waitForScanEvent(resourceURI, afterEtag) {
    if (afterEtag === void 0) { afterEtag = null; }
    return __awaiter(this, void 0, void 0, function () {
        var eventTable, acceptedScanEvent, currentEtag;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    console.log("Start listening for new ScanEvent");
                    return [4 /*yield*/, HPApi_1.default.getEvents(afterEtag !== null && afterEtag !== void 0 ? afterEtag : "")];
                case 1:
                    eventTable = _a.sent();
                    acceptedScanEvent = null;
                    currentEtag = eventTable.etag;
                    _a.label = 2;
                case 2:
                    if (!(acceptedScanEvent == null)) return [3 /*break*/, 4];
                    return [4 /*yield*/, HPApi_1.default.getEvents(currentEtag, 1200)];
                case 3:
                    eventTable = _a.sent();
                    currentEtag = eventTable.etag;
                    acceptedScanEvent = eventTable.eventTable.events.find(function (ev) {
                        return ev.isScanEvent &&
                            ev.destinationURI &&
                            ev.destinationURI.indexOf(resourceURI) >= 0;
                    });
                    return [3 /*break*/, 2];
                case 4: return [2 /*return*/, acceptedScanEvent];
            }
        });
    });
}
function waitPrinterUntilItIsReadyToUploadOrCompleted(jobUrl) {
    return __awaiter(this, void 0, void 0, function () {
        var job, isReadyToUpload;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    job = null;
                    isReadyToUpload = false;
                    _a.label = 1;
                case 1: return [4 /*yield*/, HPApi_1.default.getJob(jobUrl)];
                case 2:
                    job = _a.sent();
                    if (job.jobState === "Canceled") {
                        return [2 /*return*/, job];
                    }
                    else if (job.pageState === "ReadyToUpload" ||
                        job.jobState === "Completed") {
                        isReadyToUpload = true;
                    }
                    else if (job.jobState == "Processing") {
                        isReadyToUpload = false;
                    }
                    else {
                        console.log("Unknown jobState: ".concat(job.jobState));
                    }
                    return [4 /*yield*/, delay(300)];
                case 3:
                    _a.sent();
                    _a.label = 4;
                case 4:
                    if (!isReadyToUpload) return [3 /*break*/, 1];
                    _a.label = 5;
                case 5: return [2 /*return*/, job];
            }
        });
    });
}
function register() {
    return __awaiter(this, void 0, void 0, function () {
        var destination, hostname, toComp, walkupScanDestinations, destinations, walkupScanDestinations, destinations, resourceURI;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    hostname = os_1.default.hostname();
                    return [4 /*yield*/, HPApi_1.default.getWalkupScanToCompCaps()];
                case 1:
                    toComp = _a.sent();
                    if (!toComp) return [3 /*break*/, 3];
                    return [4 /*yield*/, HPApi_1.default.getWalkupScanToCompDestinations()];
                case 2:
                    walkupScanDestinations = _a.sent();
                    destinations = walkupScanDestinations.destinations;
                    console.log("Host destinations fetched:", destinations.map(function (d) { return d.name; }).join(", "));
                    destination = destinations.find(function (x) { return x.name === hostname; });
                    return [3 /*break*/, 5];
                case 3: return [4 /*yield*/, HPApi_1.default.getWalkupScanDestinations()];
                case 4:
                    walkupScanDestinations = _a.sent();
                    destinations = walkupScanDestinations.destinations;
                    console.log("Host destinations fetched:", destinations.map(function (d) { return d.name; }).join(", "));
                    destination = destinations.find(function (x) { return x.name === hostname; });
                    _a.label = 5;
                case 5:
                    if (!destination) return [3 /*break*/, 6];
                    console.log("Re-using existing destination: ".concat(hostname, " - ").concat(destination.resourceURI));
                    resourceURI = destination.resourceURI;
                    return [3 /*break*/, 8];
                case 6: return [4 /*yield*/, HPApi_1.default.registerDestination(new Destination_1.default(hostname, hostname, toComp), toComp)];
                case 7:
                    resourceURI = _a.sent();
                    console.log("New Destination registered: ".concat(hostname, " - ").concat(resourceURI));
                    _a.label = 8;
                case 8: return [2 /*return*/, resourceURI];
            }
        });
    });
}
function TryGetDestination(event) {
    return __awaiter(this, void 0, void 0, function () {
        var destination, i, destinationURI, shortcut;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    destination = null;
                    i = 0;
                    _a.label = 1;
                case 1:
                    if (!(i < 20)) return [3 /*break*/, 7];
                    destinationURI = event.destinationURI;
                    if (!destinationURI) return [3 /*break*/, 3];
                    return [4 /*yield*/, HPApi_1.default.getDestination(destinationURI)];
                case 2:
                    destination = _a.sent();
                    shortcut = destination.shortcut;
                    if (shortcut !== "") {
                        return [2 /*return*/, destination];
                    }
                    return [3 /*break*/, 4];
                case 3:
                    console.log("No destination URI found");
                    _a.label = 4;
                case 4:
                    console.log("No shortcut yet available, attempt: ".concat(i + 1, "/20"));
                    return [4 /*yield*/, new Promise(function (resolve) { return setTimeout(resolve, 1000); })];
                case 5:
                    _a.sent(); //wait 1s
                    _a.label = 6;
                case 6:
                    i++;
                    return [3 /*break*/, 1];
                case 7:
                    console.log("Failing to detect destination shortcut");
                    console.log(JSON.stringify(destination));
                    return [2 /*return*/, null];
            }
        });
    });
}
function fixJpegSize(filePath) {
    return __awaiter(this, void 0, void 0, function () {
        var buffer, height;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, promises_1.default.readFile(filePath)];
                case 1:
                    buffer = _a.sent();
                    height = JpegUtil_1.default.fixSizeWithDNL(buffer);
                    if (!(height != null)) return [3 /*break*/, 3];
                    // rewrite the fixed file
                    return [4 /*yield*/, promises_1.default.writeFile(filePath, buffer)];
                case 2:
                    // rewrite the fixed file
                    _a.sent();
                    return [2 /*return*/, height];
                case 3: return [2 /*return*/, null];
            }
        });
    });
}
function createScanPage(job, currentPageNumber, filePath, sizeFixed) {
    var height = sizeFixed !== null && sizeFixed !== void 0 ? sizeFixed : job.imageHeight;
    return {
        path: filePath,
        pageNumber: currentPageNumber,
        width: job.imageWidth,
        height: height,
        xResolution: job.xResolution,
        yResolution: job.yResolution,
    };
}
function handleProcessingState(job, inputSource, folder, scanCount, currentPageNumber) {
    return __awaiter(this, void 0, void 0, function () {
        var destinationFilePath, filePath, sizeFixed;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!(job.pageState == "ReadyToUpload" &&
                        job.binaryURL != null &&
                        job.currentPageNumber != null)) return [3 /*break*/, 4];
                    console.log("Ready to download page job page ".concat(job.currentPageNumber, " at:"), job.binaryURL);
                    destinationFilePath = PathHelper_1.default.getFileForPage(folder, scanCount, currentPageNumber, program.opts().pattern, "jpg");
                    return [4 /*yield*/, HPApi_1.default.downloadPage(job.binaryURL, destinationFilePath)];
                case 1:
                    filePath = _a.sent();
                    console.log("Page downloaded to:", filePath);
                    sizeFixed = null;
                    if (!(inputSource == "Adf")) return [3 /*break*/, 3];
                    return [4 /*yield*/, fixJpegSize(filePath)];
                case 2:
                    sizeFixed = _a.sent();
                    if (sizeFixed == null) {
                        console.log("File size has not been fixed, DNF may not have been found and approximate height is: ".concat(job.imageHeight));
                    }
                    _a.label = 3;
                case 3: return [2 /*return*/, createScanPage(job, currentPageNumber, filePath, sizeFixed)];
                case 4:
                    console.log("Unknown pageState: ".concat(job.pageState));
                    return [4 /*yield*/, delay(200)];
                case 5:
                    _a.sent();
                    return [2 /*return*/, null];
            }
        });
    });
}
function waitScanRequest(compEventURI) {
    return __awaiter(this, void 0, void 0, function () {
        var waitMax, i, walkupScanToCompEvent, message;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    waitMax = 50;
                    i = 0;
                    _a.label = 1;
                case 1:
                    if (!(i < waitMax)) return [3 /*break*/, 5];
                    return [4 /*yield*/, HPApi_1.default.getWalkupScanToCompEvent(compEventURI)];
                case 2:
                    walkupScanToCompEvent = _a.sent();
                    message = walkupScanToCompEvent.eventType;
                    if (message === "HostSelected") {
                        // this ok to wait
                    }
                    else if (message === "ScanRequested") {
                        return [3 /*break*/, 5];
                    }
                    else if (message === "ScanNewPageRequested") {
                        return [3 /*break*/, 5];
                    }
                    else if (message === "ScanPagesComplete") {
                        console.log("no more page to scan, scan is finished");
                        return [2 /*return*/, false];
                    }
                    else {
                        console.log("Unknown eventType: ".concat(message));
                        return [2 /*return*/, false];
                    }
                    console.log("Waiting user input: ".concat(i + 1, "/").concat(waitMax));
                    return [4 /*yield*/, new Promise(function (resolve) { return setTimeout(resolve, 1000); })];
                case 3:
                    _a.sent(); //wait 1s
                    _a.label = 4;
                case 4:
                    i++;
                    return [3 /*break*/, 1];
                case 5: return [2 /*return*/, true];
            }
        });
    });
}
function executeScanJob(scanJobSettings, inputSource, folder, scanCount, scanJobContent) {
    return __awaiter(this, void 0, void 0, function () {
        var jobUrl, job, page;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, HPApi_1.default.postJob(scanJobSettings)];
                case 1:
                    jobUrl = _a.sent();
                    console.log("New job created:", jobUrl);
                    return [4 /*yield*/, HPApi_1.default.getJob(jobUrl)];
                case 2:
                    job = _a.sent();
                    _a.label = 3;
                case 3:
                    if (!(job.jobState !== "Completed")) return [3 /*break*/, 10];
                    return [4 /*yield*/, waitPrinterUntilItIsReadyToUploadOrCompleted(jobUrl)];
                case 4:
                    job = _a.sent();
                    if (job.jobState == "Completed") {
                        return [3 /*break*/, 3];
                    }
                    if (!(job.jobState === "Processing")) return [3 /*break*/, 6];
                    return [4 /*yield*/, handleProcessingState(job, inputSource, folder, scanCount, scanJobContent.elements.length + 1)];
                case 5:
                    page = _a.sent();
                    if (page != null) {
                        scanJobContent.elements.push(page);
                    }
                    return [3 /*break*/, 9];
                case 6:
                    if (!(job.jobState === "Canceled")) return [3 /*break*/, 7];
                    console.log("Job cancelled by device");
                    return [3 /*break*/, 10];
                case 7:
                    console.log("Unhandled jobState: ".concat(job.jobState));
                    return [4 /*yield*/, delay(200)];
                case 8:
                    _a.sent();
                    _a.label = 9;
                case 9: return [3 /*break*/, 3];
                case 10:
                    console.log("Job state: ".concat(job.jobState, ", totalPages: ").concat(job.totalPageNumber, ":"));
                    return [2 /*return*/];
            }
        });
    });
}
function waitScanNewPageRequest(compEventURI) {
    return __awaiter(this, void 0, void 0, function () {
        var startNewScanJob, wait, walkupScanToCompEvent, message;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    startNewScanJob = false;
                    wait = true;
                    _a.label = 1;
                case 1:
                    if (!wait) return [3 /*break*/, 4];
                    return [4 /*yield*/, new Promise(function (resolve) { return setTimeout(resolve, 1000); })];
                case 2:
                    _a.sent(); //wait 1s
                    return [4 /*yield*/, HPApi_1.default.getWalkupScanToCompEvent(compEventURI)];
                case 3:
                    walkupScanToCompEvent = _a.sent();
                    message = walkupScanToCompEvent.eventType;
                    if (message === "ScanNewPageRequested") {
                        startNewScanJob = true;
                        wait = false;
                    }
                    else if (message === "ScanPagesComplete") {
                        wait = false;
                    }
                    else if (message === "ScanRequested") {
                        // continue waiting
                    }
                    else {
                        wait = false;
                        console.log("Unknown eventType: ".concat(message));
                    }
                    return [3 /*break*/, 1];
                case 4: return [2 /*return*/, startNewScanJob];
            }
        });
    });
}
function executeScanJobs(scanJobSettings, inputSource, folder, scanCount, scanJobContent, firstEvent) {
    return __awaiter(this, void 0, void 0, function () {
        var lastEvent, startNewScanJob;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, executeScanJob(scanJobSettings, inputSource, folder, scanCount, scanJobContent)];
                case 1:
                    _a.sent();
                    lastEvent = firstEvent;
                    if (!(lastEvent.compEventURI &&
                        inputSource !== "Adf" &&
                        lastEvent.destinationURI)) return [3 /*break*/, 8];
                    return [4 /*yield*/, waitForScanEvent(lastEvent.destinationURI, lastEvent.agingStamp)];
                case 2:
                    lastEvent = _a.sent();
                    if (!lastEvent.compEventURI) {
                        return [2 /*return*/];
                    }
                    return [4 /*yield*/, waitScanNewPageRequest(lastEvent.compEventURI)];
                case 3:
                    startNewScanJob = _a.sent();
                    _a.label = 4;
                case 4:
                    if (!startNewScanJob) return [3 /*break*/, 8];
                    return [4 /*yield*/, executeScanJob(scanJobSettings, inputSource, folder, scanCount, scanJobContent)];
                case 5:
                    _a.sent();
                    if (!lastEvent.destinationURI) {
                        return [3 /*break*/, 8];
                    }
                    return [4 /*yield*/, waitForScanEvent(lastEvent.destinationURI, lastEvent.agingStamp)];
                case 6:
                    lastEvent = _a.sent();
                    if (!lastEvent.compEventURI) {
                        return [2 /*return*/];
                    }
                    return [4 /*yield*/, waitScanNewPageRequest(lastEvent.compEventURI)];
                case 7:
                    startNewScanJob = _a.sent();
                    return [3 /*break*/, 4];
                case 8: return [2 /*return*/];
            }
        });
    });
}
function mergeToPdf(folder, scanCount, scanJobContent) {
    return __awaiter(this, void 0, void 0, function () {
        var pdfFilePath;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    pdfFilePath = PathHelper_1.default.getFileForScan(folder, scanCount, program.opts().pattern, "pdf");
                    return [4 /*yield*/, (0, ScanContent_1.createPdfFrom)(scanJobContent, pdfFilePath)];
                case 1:
                    _a.sent();
                    scanJobContent.elements.forEach(function (e) { return promises_1.default.unlink(e.path); });
                    return [2 /*return*/, pdfFilePath];
            }
        });
    });
}
function displayPdfScan(pdfFilePath, scanJobContent) {
    console.log("The following page(s) have been rendered inside '".concat(pdfFilePath, "': "));
    scanJobContent.elements.forEach(function (e) {
        return console.log("\t- page ".concat(e.pageNumber.toString().padStart(3, " "), " - ").concat(e.width, "x").concat(e.height));
    });
}
function displayJpegScan(scanJobContent) {
    scanJobContent.elements.forEach(function (e) {
        return console.log("\t- page ".concat(e.pageNumber.toString().padStart(3, " "), " - ").concat(e.width, "x").concat(e.height, " - ").concat(e.path));
    });
}
function saveScan(event, folder, scanCount) {
    return __awaiter(this, void 0, void 0, function () {
        var proceedToScan, destination, contentType, toPdf, scanStatus, inputSource, scanJobSettings, scanJobContent, pdfFilePath;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!event.compEventURI) return [3 /*break*/, 2];
                    return [4 /*yield*/, waitScanRequest(event.compEventURI)];
                case 1:
                    proceedToScan = _a.sent();
                    if (!proceedToScan) {
                        return [2 /*return*/];
                    }
                    _a.label = 2;
                case 2: return [4 /*yield*/, TryGetDestination(event)];
                case 3:
                    destination = _a.sent();
                    if (!destination) {
                        console.log("No shortcut selected!");
                        return [2 /*return*/];
                    }
                    console.log("Selected shortcut: " + destination.shortcut);
                    contentType = destination.getContentType();
                    toPdf = destination.shortcut === "SavePDF" || destination.shortcut === "EmailPDF";
                    return [4 /*yield*/, HPApi_1.default.getScanStatus()];
                case 4:
                    scanStatus = _a.sent();
                    console.log("Afd is : " + scanStatus.adfState);
                    inputSource = scanStatus.getInputSource();
                    scanJobSettings = new ScanJobSettings_1.default(inputSource, contentType);
                    scanJobContent = { elements: [] };
                    return [4 /*yield*/, executeScanJobs(scanJobSettings, inputSource, folder, scanCount, scanJobContent, event)];
                case 5:
                    _a.sent();
                    console.log("Scan of page(s) completed totalPages: ".concat(scanJobContent.elements.length, ":"));
                    if (!toPdf) return [3 /*break*/, 7];
                    return [4 /*yield*/, mergeToPdf(folder, scanCount, scanJobContent)];
                case 6:
                    pdfFilePath = _a.sent();
                    displayPdfScan(pdfFilePath, scanJobContent);
                    return [3 /*break*/, 8];
                case 7:
                    displayJpegScan(scanJobContent);
                    _a.label = 8;
                case 8: return [2 /*return*/];
            }
        });
    });
}
var iteration = 0;
function init() {
    return __awaiter(this, void 0, void 0, function () {
        var folder, scanCount, keepActive, errorCount, resourceURI, event_1, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, PathHelper_1.default.getOutputFolder(program.opts().directory)];
                case 1:
                    folder = _a.sent();
                    console.log("Target folder: ".concat(folder));
                    scanCount = 0;
                    keepActive = true;
                    errorCount = 0;
                    _a.label = 2;
                case 2:
                    if (!keepActive) return [3 /*break*/, 10];
                    console.log("Running iteration: ".concat(iteration, " - errorCount: ").concat(errorCount));
                    _a.label = 3;
                case 3:
                    _a.trys.push([3, 7, , 8]);
                    return [4 /*yield*/, register()];
                case 4:
                    resourceURI = _a.sent();
                    console.log("Waiting scan event for:", resourceURI);
                    return [4 /*yield*/, waitForScanEvent(resourceURI)];
                case 5:
                    event_1 = _a.sent();
                    scanCount++;
                    console.log("Scan event captured, saving scan #".concat(scanCount));
                    return [4 /*yield*/, saveScan(event_1, folder, scanCount)];
                case 6:
                    _a.sent();
                    return [3 /*break*/, 8];
                case 7:
                    e_1 = _a.sent();
                    errorCount++;
                    console.error(e_1);
                    console.log(e_1);
                    return [3 /*break*/, 8];
                case 8:
                    if (errorCount === 50) {
                        keepActive = false;
                    }
                    return [4 /*yield*/, delay(1000)];
                case 9:
                    _a.sent();
                    return [3 /*break*/, 2];
                case 10: return [2 /*return*/];
            }
        });
    });
}
function findOfficejetIp() {
    return new Promise(function (resolve) {
        var bonjour = (0, bonjour_1.default)();
        console.log("Searching printer...");
        var browser = bonjour.find({
            type: "http",
        }, function (service) {
            console.log(".");
            if (service.name.startsWith(program.opts().name) &&
                service.port === 80 &&
                service.type === "http" &&
                service.addresses != null) {
                browser.stop();
                bonjour.destroy();
                console.log("Found: ".concat(service.name));
                resolve(service.addresses[0]);
            }
        });
        browser.start();
    });
}
function main() {
    return __awaiter(this, void 0, void 0, function () {
        var ip, debug;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    program.option("-ip, --address <ip>", "IP address of the printer (this overrides -p)");
                    program.option("-n, --name <name>", "Name of the printer for service discovery", "HP Smart Tank Plus 570 series"); //or i.e. 'Deskjet 3520 series'
                    program.option("-d, --directory <dir>", "Directory where scans are saved (defaults to /tmp/scan-to-pc<random>)");
                    program.option("-p, --pattern <pattern>", 'Pattern for filename (i.e. "scan"_dd.mm.yyyy_hh:MM:ss, without this its scanPage<number>)');
                    program.option("-D, --debug", "Enable debug");
                    program.parse(process.argv);
                    ip = program.opts().address || "192.168.1.53";
                    if (!!ip) return [3 /*break*/, 2];
                    return [4 /*yield*/, findOfficejetIp()];
                case 1:
                    ip = _a.sent();
                    _a.label = 2;
                case 2:
                    debug = program.opts().debug != null;
                    HPApi_1.default.setDebug(debug);
                    HPApi_1.default.setPrinterIP(ip);
                    return [4 /*yield*/, init()];
                case 3:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    });
}
main();
//# sourceMappingURL=index.js.map